# SPDX-FileCopyrightText: 2020 2020 UMG MeDIC <marcel.parciak@med.uni-goettingen.de>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import datetime

from typing import List, Optional

from sqlalchemy.orm import Session

from uploader import database
from uploader import models_internal as intmodel


def get_db():
    db = database.SessionLocal()
    try:
        yield db
    finally:
        db.close()


def start_upload_procedure(db: Session) -> intmodel.UploadProcedure:
    upload_procedure = intmodel.UploadProcedure(state=intmodel.UploadState.started, started_at=datetime.datetime.now())
    db.add(upload_procedure)
    db.commit()
    db.refresh(upload_procedure)
    return upload_procedure


def get_uploads_by_state(db: Session, state: intmodel.UploadState) -> List[intmodel.UploadProcedure]:
    return db.query(intmodel.UploadProcedure).filter(intmodel.UploadProcedure.state == state).all()


def store_upload(db: Session, upload: intmodel.UploadProcedure) -> intmodel.UploadProcedure:
    db.add(upload)
    db.commit()
    db.refresh(upload)
    return upload


def get_upload_by_id(db: Session, id: int) -> Optional[intmodel.UploadProcedure]:
    return db.query(intmodel.UploadProcedure).filter(intmodel.UploadProcedure.id == id).first()
