from fastapi.testclient import TestClient

from uploader.main import app
from uploader import config
from requests.auth import HTTPBasicAuth
import pytest, json, os, requests, time

testclient = TestClient(app)


def test_receive():
    # ToDo: form correct ActiveWorkflow register message
    # see https://github.com/automaticmode/active_workflow/wiki/Remote-Agent-API#the-register-method
    payload = {"method": "register", "params": {}}
    response = testclient.post("/datalake", json=payload)
    assert response.status_code == 200


def test_fileUpload():
    os.makedirs("/tmp/source", exist_ok=True)
    with open("/tmp/source/test.csv", "w+") as test_file:
        test_file.write("Teststring")
    files = ["test.csv"]
    uploadDataToCdstar(files)


def test_multipleFileUpload():
    os.makedirs("/tmp/source", exist_ok=True)
    with open("/tmp/source/test1.csv", "w+") as test_file:
        test_file.write("Teststring1")

    with open("/tmp/source/test2.csv", "w+") as test_file:
        test_file.write("Teststring2")
    files = ["test1.csv", "test2.csv"]
    uploadDataToCdstar(files)


def uploadDataToCdstar(files):
    payload = {
        "method": "receive",
        "params": {
            "message": {"payload": {"files_moved": files}},
            "options": {
                "source_directory": ".",
                "source_filemask": "*.csv",
                "mime_type": "application/csv",
                "delete_after_upload": "false",
            },
            "memory": {"key": "value"},
            "credentials": [{"name": "admin_email", "value": "x@example.com"}],
        },
    }
    # upload test.csv to cdstar
    response = testclient.post("/datalake/receive", json=payload)
    responseValue = json.loads(response.content)
    cdstarId = responseValue["result"]["memory"]["uploads"][0]
    time.sleep(3)
    # check if following message results: Upload to CDSTAR for ID xy finished.
    # Assert true if the result contains "finished":
    payload = {
        "method": "check",
        "params": {
            "message": {"payload": {"files_moved": files}},
            "options": {
                "source_directory": "/tmp/source/",
                "source_filemask": "*.csv",
                "mime_type": "application/csv",
                "delete_after_upload": "false",
            },
            "memory": {"uploads": [cdstarId]},
            "credentials": [{"name": "admin_email", "value": "x@example.com"}],
        },
    }
    response = testclient.post("/datalake/check", json=payload)
    jsonCdstarFeedback = json.loads(response.content)
    cdstarFeedback = json.dumps(jsonCdstarFeedback)
    assert "finished" in cdstarFeedback
    responseValue = json.loads(response.content)

    # check if the file is equals to the testfile
    cdstarArchiveId = responseValue["result"]["messages"][0]["archive_id"]
    user = config.BasicSettings().cdstar_user
    password = config.BasicSettings().cdstar_pass
    uri = config.BasicSettings().cdstar_uri
    vault = config.BasicSettings().cdstar_vault_target
    if len(files) == 2:
        requestUrl = uri + "/" + vault + "/" + cdstarArchiveId + "/test1.csv"
        response = requests.get(requestUrl, auth=HTTPBasicAuth(user, password))
        fileContent = response.content.decode("utf-8")
        assert fileContent == "Teststring1"
        # check file 2
        requestUrl = uri + "/" + vault + "/" + cdstarArchiveId + "/test2.csv"
        response = requests.get(requestUrl, auth=HTTPBasicAuth(user, password))
        fileContent = response.content.decode("utf-8")
        assert fileContent == "Teststring2"
    if len(files) == 1:
        requestUrl = uri + "/" + vault + "/" + cdstarArchiveId + "/test.csv"
        response = requests.get(requestUrl, auth=HTTPBasicAuth(user, password))
        fileContent = response.content.decode("utf-8")
        assert fileContent == "Teststring"
